@@ -5,42 +5,48 @@ import numpy as np
 from moviepy.tools import cvsecs
 
 from moviepy.video.io.ffmpeg_reader import ffmpeg_parse_infos
-from moviepy.conf import FFMPEG_BINARY
+from moviepy.config import get_setting
+
+import os
+try:
+    from subprocess import DEVNULL  # py3k
+except ImportError:
+    DEVNULL = open(os.devnull, 'wb')
+
 
-    
 class FFMPEG_AudioReader:
     """
     A class to read the audio in either video files or audio files
     using ffmpeg. ffmpeg will read any audio and transform them into
     raw data.
-    
+
     Parameters
     ------------
-    
+
     filename
       Name of any video or audio file, like ``video.mp4`` or
       ``sound.wav`` etc.
-      
+
     buffersize
       The size of the buffer to use. Should be bigger than the buffer
       used by ``to_audiofile``
-    
+
     print_infos
       Print the ffmpeg infos on the file being read (for debugging)
-      
+
     fps
       Desired frames per second in the decoded signal that will be
       received from ffmpeg
-      
+
     nbytes
       Desired number of bytes (1,2,4) in the signal that will be
       received from ffmpeg
-          
+
     """
-        
+
     def __init__(self, filename, buffersize, print_infos=False,
                  fps=44100, nbytes=2, nchannels=2):
-        
+
         self.filename = filename
         self.nbytes = nbytes
         self.fps = fps
@@ -55,21 +61,21 @@ class FFMPEG_AudioReader:
             self.duration = infos['duration']
         self.infos = infos
         self.proc = None
-        
+
         self.nframes = int(self.fps * self.duration)
         self.buffersize= min( self.nframes+1, buffersize )
         self.buffer= None
         self.buffer_startframe = 1
         self.initialize()
         self.buffer_around(1)
-    
-    
-    
+
+
+
     def initialize(self, starttime = 0):
         """ Opens the file, creates the pipe. """
-    
+
         self.close_proc() # if any
-        
+
         if starttime !=0 :
             offset = min(1,starttime)
             i_arg = ["-ss", "%.05f"%(starttime-offset),
@@ -77,29 +83,36 @@ class FFMPEG_AudioReader:
                     "-ss", "%.05f"%offset]
         else:
             i_arg = [ '-i', self.filename,  '-vn']
-             
-        
-        cmd = ([FFMPEG_BINARY] + i_arg + 
+
+
+        cmd = ([get_setting("FFMPEG_BINARY")] + i_arg +
                [ '-loglevel', 'error',
                  '-f', self.f,
                 '-acodec', self.acodec,
                 '-ar', "%d"%self.fps,
                 '-ac', '%d'%self.nchannels, '-'])
-        self.proc = sp.Popen( cmd, bufsize=self.buffersize,
-                                   stdout=sp.PIPE,
-                                   stderr=sp.PIPE)
-        
+
+        popen_params = {"bufsize": self.buffersize,
+                        "stdout": sp.PIPE,
+                        "stderr": sp.PIPE,
+                        "stdin": DEVNULL}
+
+        if os.name == "nt":
+            popen_params["creationflags"] = 0x08000000
+
+        self.proc = sp.Popen( cmd, **popen_params)
+
         self.pos = np.round(self.fps*starttime)
-     
-     
-     
+
+
+
     def skip_chunk(self,chunksize):
         s = self.proc.stdout.read(self.nchannels*chunksize*self.nbytes)
         self.proc.stdout.flush()
         self.pos = self.pos+chunksize
-        
-        
-        
+
+
+
     def read_chunk(self,chunksize):
         L = self.nchannels*chunksize*self.nbytes
         s = self.proc.stdout.read(L)
@@ -111,7 +124,7 @@ class FFMPEG_AudioReader:
         #self.proc.stdout.flush()
         self.pos = self.pos+chunksize
         return result
-         
+
 
 
     def seek(self,pos):
@@ -130,7 +143,7 @@ class FFMPEG_AudioReader:
             self.skip_chunk(pos-self.pos)
         # last case standing: pos = current pos
         self.pos = pos
-    
+
 
 
     def close_proc(self):
@@ -140,25 +153,25 @@ class FFMPEG_AudioReader:
                          self.proc.stderr]:
                 std.close()
             del self.proc
-    
+
     def get_frame(self, tt):
-        
+
         buffersize = self.buffersize
         if isinstance(tt,np.ndarray):
             # lazy implementation, but should not cause problems in
             # 99.99 %  of the cases
-            
-            
+
+
             # elements of t that are actually in the range of the
             # audio file.
-            
+
             in_time = (tt>=0) & (tt < self.duration)
-            
+
             # The np.round in the next line is super-important.
             # Removing it results in artifacts in the noise.
             frames = np.round((self.fps*tt)).astype(int)[in_time]
             fr_min, fr_max = frames.min(), frames.max()
-            
+
             if not (0 <=
                      (fr_min - self.buffer_startframe)
                           < len(self.buffer)):
@@ -167,7 +180,7 @@ class FFMPEG_AudioReader:
                         (fr_max - self.buffer_startframe)
                              < len(self.buffer)):
                 self.buffer_around(fr_max)
-                
+
             try:
                 result = np.zeros((len(tt),self.nchannels))
                 indices = frames - self.buffer_startframe
@@ -178,20 +191,20 @@ class FFMPEG_AudioReader:
                        "At time t=%.02f-%.02f seconds, "%(tt[0], tt[-1])+
                        "indices wanted: %d-%d, "%(indices.min(), indices.max())+
                        "but len(buffer)=%d\n"%(len(self.buffer))+ str(error))
-                
+
         else:
-            
+
             ind = int(self.fps*tt)
             if ind<0 or ind> self.nframes: # out of time: return 0
                 return np.zeros(self.nchannels)
-                
+
             if not (0 <= (ind - self.buffer_startframe) <len(self.buffer)):
                 # out of the buffer: recenter the buffer
                 self.buffer_around(ind)
-                
+
             # read the frame in the buffer
             return self.buffer[ind - self.buffer_startframe]
-                
+
 
     def buffer_around(self,framenumber):
         """
@@ -201,8 +214,8 @@ class FFMPEG_AudioReader:
 
         # start-frame for the buffer
         new_bufferstart = max(0,  framenumber - self.buffersize // 2)
-        
-        
+
+
         if (self.buffer!=None):
             current_f_end  = self.buffer_startframe + self.buffersize
             if (new_bufferstart <
@@ -219,12 +232,12 @@ class FFMPEG_AudioReader:
         else:
             self.seek(new_bufferstart)
             self.buffer =  self.read_chunk(self.buffersize)
-        
+
         self.buffer_startframe = new_bufferstart
-    
-    
+
+
     def __del__(self):
         self.close_proc()
-        
-        
+
+
 
